// static Rational with I/O capabilities
//
// Load your own tuning file with the .read(string) method
// wraps beginning at MIDI note 0 and goes to 128

// Note: You must initialize tMap with your controller size
// (not only MIDI, but others could work) using the line
// new float[128] @=> Rational.tMap; at the bottom of
// the current document. Other presets are below.

public class Rational
{
	static FileIO @ rFile;
	
	static float rFundamental; // 'A' at 'A' = 440
	
	static float rMap[];
	static int rOffSet;
	static int rDiasponOffSet;
	
	static float rBaseMap[];
	
	//	Reads in a new tuning file when given a filename string as an argument.
	public static void readTuning(string rFilename) 
	{
		
		rBaseMap.clear();
		
		rFile.open( rFilename, FileIO.READ );
		
		if ( !rFile.good() ) {
			<<< "can't open file: ", rFilename, " for reading." >>>;
			me.exit();
		}
		
		<<< "Name of tuning is: ", rFile.readLine() >>>;
		<<< "rFundamental is: ", rFundamental >>>;
		
		for ( int i; rFile.more(); i++ ) {
			rBaseMap << ( (rFile.readInt()$float) / (rFile.readInt()$float) * rFundamental );
			
			<<< "rBaseMap element ", i, " is: ", rBaseMap[i] >>>;
		}
		
		<<< rBaseMap.size() >>>;
		for ( 0 => int i; i < rMap.size(); i++ ) {
			rBaseMap[ ( ( i + rOffSet ) % rBaseMap.size() ) ] * 
			Math.pow(2, rDiasponOffSet + 
			Math.floor( ( ( i + rOffSet ) / rBaseMap.size() ) ) ) 
			=> rMap[i];
		}
	}
	
	
	public static float r(int key) 
	{
		// <<< "pitch!", tMap[key] >>>;
		return rMap[key];
	}
}

// Default settings and initializations:

13.75 => Rational.rFundamental; // Fundamental pitch (low 'A' at 'A' = 440)
new float[128] @=> Rational.rMap; // This can change the max controller size
3 => Rational.rOffSet; // Key offset: setting to 3 will use a fundamental of the 'A' key
-1 => Rational.rDiasponOffSet; // Diaspon offset: setting to -1 will keep the tuning 'A' at 440
new float[0] @=> Rational.rBaseMap; // Making the tBaseMap start with a size of 0
new FileIO @=> Rational.rFile; // Necessary for the initialization of the object
Rational.readTuning("partch.rational"); // Default: Partch's 43-tone scale
